const { watchFnName, generateFn, watchParamStr, getFnMain } = require('./commonutil')
const { ts2jsTypeMap, ts2bableTypeMap } = require('./vars')
module.exports = {
  components: {
    handler (item) {
      return {
        componentsStr: generateFn(item)
      }
    }
  },
  mixins: {
    handler (item) {
      let tsVueMixinsList = []
      item.value.elements.forEach(v => {
        tsVueMixinsList.push(generateFn(v))
      })
      return {
        tsVueOptsList: ['Mixins'],
        tsVueMixinsList
      }
    }
  },
  inject: {
    handlerArrayExpression (v) {
      return `
        @InjectReactive(${generateFn(v)}) readonly ${generateFn(v).replace(/['"]/g, '')}!: string
      `
    },
    hanlderObjectExpression (v) {
      return `
        @InjectReactive(${generateFn(v.value)}) readonly ${generateFn(v.key)}!: string
      `
    },
    handler (item) {
      let injectTemplate = ''
      if (item.value.type === 'ArrayExpression') {
        item.value.elements.forEach(v => {
          injectTemplate += this.handlerArrayExpression(v)
        })
      } else if (item.value.type === 'ObjectExpression') {
        item.value.properties.forEach(v => {
          injectTemplate += this.hanlderObjectExpression(v)
        })
      }
      return {
        template: injectTemplate,
        tsVueOptsList: ['InjectReactive']
      }
    }
  },
  provide: {
    hanlderObjectExpression (v) {
      return `
        @ProvideReactive('${v.key.name}') provide${v.key.name} = ${generateFn(v.value)}
      `
    },
    handler (item) {
      let provideTemplate = ''
      if (item.type === 'ObjectMethod') {
        // es6函数
        item.body.body[0].argument.properties.forEach(v => {
          provideTemplate += this.hanlderObjectExpression(v)
        })
      } else if (item.value.type === 'ObjectExpression') {
        item.value.properties.forEach(v => {
          provideTemplate += this.hanlderObjectExpression(v)
        })
      } else if (item.value.type === 'FunctionExpression' || item.value.type === 'ArrowFunctionExpression') {
        item.value.body.body[0].argument.properties.forEach(v => {
          provideTemplate += this.hanlderObjectExpression(v)
        })
      }
      return {
        tsVueOptsList: ['ProvideReactive'],
        template: provideTemplate
      }
    }
  },
  props: {
    handler (item) {
      let propsTemplate = ''
      item.value.properties.forEach(v => {
        let propsObj = {}
        propsTemplate += `@Prop({`
        v.value.properties.forEach(vv => {
          if (vv.key.name === 'type') {
            if (vv.value.type === 'ArrayExpression') {
              vv.value.elements.forEach(vvv => {
                propsObj[vv.key.name] = (propsObj[vv.key.name] || []).concat(generateFn(vvv))
              })
            } else {
              propsObj[vv.key.name] = generateFn(vv.value)
            }
          }
          propsTemplate += `${generateFn(vv)},`
        })
        propsTemplate += '}) readonly '
        propsTemplate += `${v.key.name}:${Array.isArray(propsObj.type) ? propsObj.type.map(typeItem => ts2jsTypeMap[typeItem]).join('|') : ts2jsTypeMap[propsObj.type] || 'any'}\n`
      })
      return {
        template: propsTemplate,
        tsVueOptsList: ['Prop']
      }
    }
  },
  data: {
    handler (item) {
      let dataTemplate = ''
      let list = null
      if (item.type === 'ObjectMethod') {
        // es6 函数 () => {}
        list = item.body.body[0].argument.properties
      } else {
        list = item.value.type === 'FunctionExpression'
          // es5 函数 function () {}
          ? item.value.body.body[0].argument.properties
          // 对象 {}
          : item.value.properties
      }
      list.forEach(v => {
        dataTemplate += `${v.key.name}:${ts2bableTypeMap[v.value.type] || 'any'} = ${generateFn(v.value)}\n`
      })
      return {
        template: dataTemplate
      }
    },
  },
  model: {
    handler (item, vueExportDefaultChildren) {
      let modeTemplate = ''
      modeTemplate += `@Model(`
      const modelObj = {}
      item.value.properties.forEach(v => {
        modelObj[v.key.name] = v.value.value
      })
      modeTemplate += `'${modelObj.event || ""}'`
      if (modelObj.prop) {
        const propsObj = vueExportDefaultChildren.filter(v => v.key.name === 'props')[0]
        if (propsObj) {
          const modelPropIndex = propsObj.value.properties.findIndex(v => v.key.name === modelObj.prop)
          if (modelPropIndex !== -1) {
            const modelProp = propsObj.value.properties[modelPropIndex]
            // 此 prop 将在 model 中定义，因此需要从原 prop 中删去
            propsObj.value.properties.splice(modelPropIndex, 1)
            const modelOpts = {}
            modeTemplate += `, {`
            modelProp.value.properties.forEach(v => {
              modelOpts[v.key.name] = generateFn(v.value)
              modeTemplate += `${v.key.name}: ${generateFn(v.value)},`
            })
            modeTemplate += `}) readonly ${modelProp.key.name}!: ${ts2jsTypeMap[modelOpts.type] || 'any'}\n`
          }
        }
      }
      return {
        template: modeTemplate,
        tsVueOptsList: ['Model']
      }
    }
  },
  watch: {
    handlerStringLiteral (v, value, kk) {
      return `
        @Watch('${watchParamStr(v.key)}')
        on${watchFnName(generateFn(v.key))}${kk === void 0 ? '' : kk}Changed (val, oldVal) {
          this[${generateFn(value)}](val, oldVal)
        }
      `
    },
    handlerES6FunctionExpression (v) {
      return `
        @Watch('${watchParamStr(v.key)}')
        on${watchFnName(generateFn(v.key))}Changed ${generateFn(v).replace(/[\s\S]+?(?=\()/, '')}
      `
    },
    handlerArrowFunctionExpression (v, value, kk) {
      return `
        @Watch('${watchParamStr(v.key)}')
        on${watchFnName(generateFn(v.key))}${kk === void 0 ? '' : kk}Changed ${generateFn(value).replace('=>', '')}
      `
    },
    handlerFunctionExpression (v, value, kk) {
      return `
        @Watch('${watchParamStr(v.key)}')
        on${watchFnName(generateFn(v.key))}${kk === void 0 ? '' : kk}Changed ${getFnMain(generateFn(value))}
      `
    },
    handlerObject (v) {
      let opts = ''
      let fn = ''
      v.properties.forEach(vv => {
        if (generateFn(vv.key) === 'handler') {
          fn += (
            vv.type === 'ObjectMethod'
              ? generateFn(vv).replace(/[\s\S]+?(?=\()/, '').replace(/}\s*$/, '')
              : (
                vv.value.type === 'FunctionExpression' || vv.value.type === 'ArrowFunctionExpression'
                  ? `${generateFn(vv.value).replace(/[\s\S]+?(?=\()/, '').replace(/}\s*$/, '').replace('=>', '')}`
                  : `(val, oldVal){ this[${generateFn(vv.value)}](val, oldVal)`
              )
          )
        } else {
          // 不是handler，那么就是 deep, immediate
          opts = opts + (opts ? ',' : ',{') + `${generateFn(vv.key)}:${generateFn(vv.value)}`
        }
      })
      return {
        opts: opts + (opts ? '})' : ')'),
        value: fn + '}'
      }
    },
    handlerObjectExpression (v, value, kk) {
      const obj = this.handlerObject(value)
      return `
        @Watch('${watchParamStr(v.key)}'${obj.opts}
        on${watchFnName(generateFn(v.key))}${kk === void 0 ? '' : kk}Changed ${obj.value}
      `
    },
    handler (item) {
      let watchTemplate = ''
      item.value.properties.forEach(v => {
        if (v.type === 'ObjectMethod') {
          // es6 函数 () {}
          watchTemplate += this.handlerES6FunctionExpression(v)
        } else if (v.value.type === 'FunctionExpression') {
          // 函数 function () {}
          watchTemplate += this.handlerFunctionExpression(v, v.value)
        } else if (v.value.type === 'ArrowFunctionExpression') {
          // 箭头函数
          watchTemplate += this.handlerArrowFunctionExpression(v, v.value)
        } else if (v.value.type === 'StringLiteral') {
          // 方法名
          watchTemplate += this.handlerStringLiteral(v, v.value)
        } else if (v.value.type === 'ObjectExpression') {
          // 对象类型
          watchTemplate += this.handlerObjectExpression(v, v.value)
        } else if (v.value.type === 'ArrayExpression') {
          // 数组
          v.value.elements.forEach((vv, kk) => {
            if (vv.type === 'StringLiteral') {
              // 方法名
              watchTemplate += this.handlerStringLiteral(v, vv, kk)
            } else if (vv.type === 'FunctionExpression') {
              // 函数 function () {}
              watchTemplate += this.handlerFunctionExpression(v, vv, kk)
            } else if (vv.type === 'ArrowFunctionExpression') {
              watchTemplate += this.handlerArrowFunctionExpression(v, vv, kk)
            } else if (vv.type === 'ObjectExpression') {
              // 对象类型
              watchTemplate += this.handlerObjectExpression(v, vv, kk)
            }
          })
        }
      })
      return {
        template: watchTemplate,
        tsVueOptsList: ['Watch']
      }
    }
  },
  computed: {
    handlerFunctionExpression (v, vv) {
      if (vv.type === 'ObjectMethod' || vv.value.type === 'FunctionExpression') {
        // es6函数 和 es5函数
        return `${generateFn(v.key)} ${getFnMain(generateFn(vv.type === 'ObjectMethod' ? vv : vv.value))}\n`
      } else if (vv.value.type === 'ArrowFunctionExpression') {
        // 箭头函数
        return `${generateFn(v.key)} (${vv.value.params.map(vvv => generateFn(vvv)).join(',')}) ${getFnMain(generateFn(vv.value.body))}\n`
      }
    },
    handler (item) {
      let computedTemplate = ''
      item.value.properties.forEach(v => {
        if (v.value && v.value.type === 'ObjectExpression') {
          // 对象形式
          v.value.properties.forEach(vv => {
            computedTemplate += (generateFn(vv.key) + ' ' + this.handlerFunctionExpression(v, vv))
          })
        } else {
          computedTemplate += ('get ' + this.handlerFunctionExpression(v, v))
        }
      })
      return {
        template: computedTemplate
      }
    }
  },
  methods: {
    handler (item) {
      let methodsTemplate = ''
      item.value.properties.forEach(v => {
        methodsTemplate += generateFn(v)
      })
      return {
        template: methodsTemplate
      }
    }
  }
}
